rem ************************************************************************
rem * uninstall-headers.bat - generated by gendsp.pl
rem *
set msvc=%1
del %COINDIR%\include\Inventor\Qt\devices\SoQtDevice.h
del %COINDIR%\include\Inventor\Qt\devices\SoQtKeyboard.h
del %COINDIR%\include\Inventor\Qt\devices\SoQtMouse.h
del %COINDIR%\include\Inventor\Qt\editors\SoQtColorEditor.h
del %COINDIR%\include\Inventor\Qt\editors\SoQtMaterialEditor.h
del %COINDIR%\include\Inventor\Qt\nodes\SoGuiColorEditor.h
del %COINDIR%\include\Inventor\Qt\nodes\SoGuiMaterialEditor.h
del %COINDIR%\include\Inventor\Qt\viewers\SoQtViewer.h
del %COINDIR%\include\Inventor\Qt\viewers\SoQtFullViewer.h
del %COINDIR%\include\Inventor\Qt\viewers\SoQtExaminerViewer.h
del %COINDIR%\include\Inventor\Qt\viewers\SoQtPlaneViewer.h
del %COINDIR%\include\Inventor\Qt\viewers\SoQtConstrainedViewer.h
del %COINDIR%\include\Inventor\Qt\viewers\SoQtFlyViewer.h
del %COINDIR%\include\Inventor\Qt\widgets\SoQtThumbWheel.h
del %COINDIR%\include\Inventor\Qt\widgets\SoQtPopupMenu.h
del %COINDIR%\include\Inventor\Qt\SoQt.h
del %COINDIR%\include\Inventor\Qt\SoQtBasic.h
del %COINDIR%\include\Inventor\Qt\SoQtObject.h
del %COINDIR%\include\Inventor\Qt\SoQtCursor.h
del %COINDIR%\include\Inventor\Qt\SoQtComponent.h
del %COINDIR%\include\Inventor\Qt\SoQtGLWidget.h
del %COINDIR%\include\Inventor\Qt\SoQtColorEditor.h
del %COINDIR%\include\Inventor\Qt\SoQtRenderArea.h
