del src\Inventor\Qt\moc_SoQtComponentP.icc
del src\Inventor\Qt\moc_SoQtGLWidgetP.icc
del src\Inventor\Qt\moc_SoQtP.icc
del src\Inventor\Qt\viewers\moc_SoQtExaminerViewerP.icc
del src\Inventor\Qt\viewers\moc_SoQtFullViewerP.icc
del src\Inventor\Qt\viewers\moc_SoQtPlaneViewerP.icc
del src\Inventor\Qt\widgets\moc_QtNativePopupMenu.icc
del src\Inventor\Qt\widgets\moc_SoQtGLArea.icc
del src\Inventor\Qt\widgets\moc_SoQtThumbWheel.icc
